#include <boost/asio.hpp> /// just including this requires to link with boost system :-(

#include <iostream>
#include <thread>


/// we post tasks (handlers) which then get dispatched and executed
/// pending versus ready handlers

/**
    In general handlers are associated which background tasks (run in the OS), the handler
    is meant to be invoked when that associated task is completed. Till that time the handler is pending,
    and then it becomes ready and can be executed.
*/


namespace
{

void doNothing()
{
    boost::asio::io_service ioService;
    ioService.run();

    std::cout << "just did nothing" << std::endl;
    std::cout << std::endl;
}

void task1()
{
    std::cout << " task 1 starting" << std::endl;

    std::this_thread::sleep_for(std::chrono::seconds(1));
    std::cout << " task 1 still busy" << std::endl;
    std::this_thread::sleep_for(std::chrono::seconds(1));

    std::cout << " task 1 finished" << std::endl;
    std::cout <<  std::endl;
}

void task2()
{
    std::cout << " task 2 starting" << std::endl;

    std::this_thread::sleep_for(std::chrono::seconds(1));
    std::cout << " task 2 still busy" << std::endl;
    std::this_thread::sleep_for(std::chrono::seconds(1));

    std::cout << " task 2 finished" << std::endl;
    std::cout <<  std::endl;
}


void postTasks()
{
    boost::asio::io_service ioService;

    std::cout << "Posting 2 tasks to the service." << std::endl;
    ioService.post(task1);
    ioService.post(task2);

    std::cout << "sleeping for 5 seconds and since we did not call run() no task(s) are processed." << std::endl;
    std::this_thread::sleep_for(std::chrono::seconds(5));

    std::cout << "sleeping done let's call run() and have the party started." << std::endl;
    ioService.run();

    std::cout << "run() returned, which means all tasks have finished." << std::endl;
    std::cout <<  std::endl;
}

} // namespace

int main()
{
    doNothing();

    postTasks();
}
