#include <boost/asio.hpp> /// just including this requires to link with boost system :-(

#include <iostream>
#include <thread>

/// we want to stop before all taks have completed
/// we issue the STOP while one task is still running
/// and while there are still other tasks scheduled


namespace
{

void task1()
{
    std::cout << " task 1 starting" << std::endl;

    std::this_thread::sleep_for(std::chrono::seconds(3));
    std::cout << " task 1 still busy" << std::endl;
    std::this_thread::sleep_for(std::chrono::seconds(3));

    std::cout << " task 1 finished" << std::endl;
    std::cout <<  std::endl;
}

void task2()
{
    std::cout << " task 2 starting" << std::endl;

    std::this_thread::sleep_for(std::chrono::seconds(1));
    std::cout << " task 2 still busy" << std::endl;
    std::this_thread::sleep_for(std::chrono::seconds(1));

    std::cout << " task 2 finished" << std::endl;
    std::cout <<  std::endl;
}


void postTasks(boost::asio::io_service& ioService)
{
    std::cout << "Posting 2 tasks to the service." << std::endl;
    ioService.post(task1);
    ioService.post(task2);
    ioService.run();

    std::cout << "run() returned, which means all tasks have finished." << std::endl;
    std::cout <<  std::endl;
}

} // namespace

int main()
{
    boost::asio::io_service ioService;

    std::cout << "let's have the ioservice being run in a dedicated thread so our main thread remains available for other work" << std::endl;
    std::thread t{postTasks, std::ref(ioService)};

    for(auto idx = 0; idx < 2; ++idx)
    {
        std::cout << "heavy work being done in main thread."  << std::endl;
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }

    std::cout << "oops, we need to go home, abort abort abort." << std::endl;
    ioService.stop();

    t.join();

    std::cout << "all have finished, time to go home." << std::endl;
    std::cout <<  std::endl;
}
