#include <boost/asio.hpp>

#include <iostream>
#include <thread>
#include <functional>

namespace
{

void task(int i)
{
    std::cout << " task execution : i = " << i << std::endl;
    std::this_thread::sleep_for(std::chrono::seconds(2));
}

void postTasks(
        boost::asio::io_context& ioContext,
        boost::asio::io_context::strand& strand)
{
    for(int i = 0; i < 10; ++i)
    {
        if(i % 2)
        {
            boost::asio::post(ioContext, std::bind(task, i));
        }
        else
        {
            boost::asio::post(strand, std::bind(task, i));
            /// deprecated strand.post(std::bind(task, i)); but preferred over ioContext.post(strand.wrap(std::bind(task, i)))
        }
    }
}

void workerThread(boost::asio::io_context& ioContext)
{
    ioContext.run();
}

} // namespace

int main()
{
    {
        boost::asio::io_context ioContext;
        boost::asio::io_context::strand strand{ioContext};

        postTasks(ioContext, strand);

        std::cout << "ALL tasks by 1 io service processed by the multiple threads" << std::endl;
        std::thread t1{workerThread, std::ref(ioContext)};
        std::thread t2{workerThread, std::ref(ioContext)};
        std::thread t3{workerThread, std::ref(ioContext)};
        t1.join();
        t2.join();
        t3.join();
    }
}
