#include <iostream>
#include <functional>
#include <thread>
#include <atomic>
#include <signal.h>
#include <boost/asio/signal_set.hpp>


/// connect async handlers to signals

/// 1 thread
/// for the io service
/// to deal with the signal (sognal to stop the application)
/// while main keeps on doing interesting stuff

namespace
{

std::atomic_bool stop{false};

void handler(
        const boost::system::error_code& /*erc*/,
        int signal)
{
    std::cout << "signal " << signal << " occurred. Time to stop the application." <<std::endl;
    stop = true;
}

void asyncSignal()
{
    boost::asio::io_context ioContext;
    boost::asio::signal_set signals{ioContext, SIGINT, SIGTERM};

    signals.async_wait(handler);
    ioContext.run();
}

} // namespace


int main()
{
    std::thread t{asyncSignal};

    while(!stop)
    {
        std::cout << "doing interesting stuff in the main thread\n";
        std::this_thread::sleep_for(std::chrono::seconds(2));
    }

    t.join();
}
