#include <iostream>
#include <functional>
#include <chrono>
#include <boost/asio.hpp>

/// task is now something doing/triggering/waiting for IO
/// this can be : bytes coming in on a socket, a timer that expires

/// so the "waiting part" can be seen as a task to be carried out by the io-service
/// and when the waiting is over, aka the event occurred/triggered, then a
/// continuation/specified callback is carried out
/// which is also to be seen as part of the task

/// Timer ==> waiting on its expiration ==> task for the io-service
///         ===> callback/handler when expired

/// waiting on the timer : synchronous ==> wait() ==> no need to call run on the ioService
/// waiting on the timer : asynchronous ==> async_wait() ===> we need to call run (in some thread) now for the io service

namespace
{
void syncTimer()
{
    boost::asio::io_service ioService;
    boost::asio::deadline_timer timer{ioService, boost::posix_time::seconds(5)}; /// :-(  does not work with std::chrono

    std::cout << "gonna wait 5 seconds by means of a synchronous timer wait\n";

    timer.wait();

    std::cout << "just waited 5 seconds by means of a synchronous timer wait\n";
}

void handler(const boost::system::error_code& erc)
{
    std::cout << "just waited 5 seconds by means of a asynchronous timer wait\n";
    std::cout << "errcode : " << erc << std::endl;
}

void asyncTimer()
{
    boost::asio::io_service ioService;
    boost::asio::deadline_timer timer{ioService, boost::posix_time::seconds(5)};

    std::cout << "gonna wait 5 seconds by means of a asynchronous timer wait\n";

    timer.async_wait(handler);

    /// sleep(10); /// even if we would sleep long enough the handler will not be called unless we call the blocking run()

    ioService.run();

    /// if we would sleep for 10 secs, and then call the run, then the handler gets called immed, but still 5 sec too late
    /// so not very interesting ...
}

} // namespace


int main()
{
    syncTimer();

    std::cout << std::endl << std::endl;

    asyncTimer();
}
