#include <iostream>
#include <functional>
#include <boost/asio/steady_timer.hpp>

/// inside the handler of a (asynchronous) timer we can "re-arm / re-configure" the timer
/// ==> bind the timer into the handler

namespace
{

void handlerRepeated(
        const boost::system::error_code& /*ec*/,
        boost::asio::steady_timer& timer,
        int& count)
{
    if(count < 4)
    {
        std::cout << count << "\n";
        ++count;
//        timer.expires_at(timer.expires_at() + std::chrono::seconds(2));  /// this is what to use on the deadline_timer
        timer.expires_after(std::chrono::seconds(2));    /// expires_from_now is deprecated() but could also be used
        timer.async_wait(std::bind(handlerRepeated, std::placeholders::_1, std::ref(timer), std::ref(count)));
    }
}

void asyncTimerRepeated()
{
    /// we will time 4 times and then we stop
    /// so we have to keep the ioservice busy ==> in the handler launch another timer cycle
    boost::asio::io_context ioContext;
    boost::asio::steady_timer timer{ioContext, std::chrono::seconds(2)};

    int count = 0;
    timer.async_wait(std::bind(handlerRepeated, std::placeholders::_1, std::ref(timer), std::ref(count)));

    ioContext.run();
}

} // namespace


int main()
{
    asyncTimerRepeated();
}
