#include <iostream>
#include <functional>
#include <thread>
#include <boost/asio/steady_timer.hpp>

/// let's handle the timer in separate thread so main thread can continue doing other stuff
/// and when timer is expired, it means main thread is allowed to stop and we can go home

namespace
{
void handler(const boost::system::error_code&, bool& stop )
{
    std::cout << "timer has expired, time to go home" << std::endl;
    stop = true;
}

void asyncTimer(bool& stop)
{
    boost::asio::io_context ioContext;
    boost::asio::steady_timer timer{ioContext, std::chrono::seconds(5)};
    timer.async_wait(std::bind(handler, std::placeholders::_1, std::ref(stop)));
    ioContext.run();
}

} // namespace

int main()
{
    bool stop{false};
    std::thread t{asyncTimer, std::ref(stop)};
    while(!stop)
    {
        std::cout << "doing interesting stuff in the main thread, till the timer will go off\n";
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }
    t.join();
    std::cout << "going home\n";
}
