#include <iostream>
#include <functional>
#include <thread>
#include <boost/asio/steady_timer.hpp>

/// 1 thread
/// for the io service
/// to deal with multiple timers
/// while main keeps on doing interesting stuff

/// should we cancel the timers, maybe just stopping the ioContext is rather brutal

namespace
{
void handler(
        const boost::system::error_code&,
        boost::asio::steady_timer& timer,
        int timerId,
        int timeOut)
{
    std::cout << "timer " << timerId << " has expired." << std::endl;
    timer.expires_after(std::chrono::seconds(timeOut));
    timer.async_wait(std::bind(handler, std::placeholders::_1, std::ref(timer), timerId, timeOut));
}

void asyncTimer(boost::asio::io_context& ioContext)
{
    boost::asio::steady_timer timer1{ioContext, std::chrono::seconds(5)};
    boost::asio::steady_timer timer2{ioContext, std::chrono::seconds(2)};
    boost::asio::steady_timer timer3{ioContext, std::chrono::seconds(3)};

    timer1.async_wait(std::bind(handler, std::placeholders::_1, std::ref(timer1), 1, 5));
    timer2.async_wait(std::bind(handler, std::placeholders::_1, std::ref(timer2), 2, 2));
    timer3.async_wait(std::bind(handler, std::placeholders::_1, std::ref(timer3), 3, 3));
    ioContext.run();
}

} // namespace

int main()
{
    boost::asio::io_context ioContext;
    std::thread t{asyncTimer, std::ref(ioContext)};
    for(auto idx = 0; idx < 10; ++idx)
    {
        std::cout << "doing interesting stuff in the main thread\n";
        std::this_thread::sleep_for(std::chrono::seconds(2));
    }
    ioContext.stop();
    t.join();
    std::cout << "going home\n";
}
