#include <iostream>
#include <functional>
#include <thread>
#include <boost/asio/steady_timer.hpp>


/// 1 thread
/// for the io service
/// to deal with the timer
/// while main keeps on doing interesting stuff
/// main decides to stop ==> cancels the timer ==> service runs out ==> thread ends

namespace
{
void handler(const boost::system::error_code& erc)
{
    std::cout << "timer has expired (or better cancelled in this case)." << std::endl;
    std::cout << "errcode : " << erc << std::endl;
    if(erc.value() == ECANCELED)
    {
        std::cout << " this is indeed the ECANCELED return code." << std::endl;
    }
    if(erc == boost::asio::error::operation_aborted)
    {
        std::cout << " this is indeed the boost::asio::error::operation_aborted return code." << std::endl;
    }
}

void asyncTimer(
    boost::asio::io_context& ioContext,
    boost::asio::steady_timer& timer)
{
    timer.async_wait(std::bind(handler, std::placeholders::_1));
    ioContext.run();
}

} // namespace


int main()
{
    boost::asio::io_context ioContext;
    boost::asio::steady_timer timer{ioContext, std::chrono::seconds(8)};

    std::thread t{asyncTimer, std::ref(ioContext), std::ref(timer)};
    for(auto idx = 0; idx < 3; ++idx)
    {
        std::cout << "doing interesting stuff in the main thread\n";
        std::this_thread::sleep_for(std::chrono::seconds(2));
    }
    std::cout << "Cancelling timer." << std::endl;
    timer.cancel();
    t.join();
}
