#include <boost/asio.hpp>

#include <thread>
#include <iostream>
#include <string>

/// we will connect to the synchronous server of Asio10-EchoServer1
/// we will make multiple threads that connect and get served

namespace
{

const boost::asio::ip::tcp::endpoint ep{boost::asio::ip::make_address("127.0.0.1"), 8242};

std::size_t completionCondition(
        std::string& buffer,
        const boost::system::error_code& /*error*/, /// let's ignore
        std::size_t bytes_transferred)
{
    if(!bytes_transferred)
    {
        return 1;
    }
    return buffer[bytes_transferred - 1] == '\n' ? 0 : 1;
}

void getServed(
        boost::asio::io_context& ioContext,
        int id)
{
    std::this_thread::sleep_for(std::chrono::seconds(1)); /// so all thread are booted up and can start to fight for the cpu

	const std::string message = "request from client " + std::to_string(id) + " .\n";

	boost::asio::ip::tcp::socket sock{ioContext};
	sock.connect(ep);
	std::cout << "client " << id << " got connected" << std::endl;

	boost::asio::write(sock, boost::asio::buffer(message));

	std::string answer;
    boost::asio::read(
            sock,
            boost::asio::dynamic_buffer(answer),
            std::bind(completionCondition, std::ref(answer), std::placeholders::_1, std::placeholders::_2));
    std::cout << "server replied : " << answer << std::endl;
    std::cout << "this is indeed echo of our mesage : " << (message == answer)<< std::endl;

	std::cout << "client " << id << " got served" << std::endl << std::endl;
	sock.close();
}

} // namespace

int main()
{
    boost::asio::io_context ioContext;


    std::vector<std::unique_ptr<std::thread>> mThreads;

    const int threads{10};

    for(auto thread = 0; thread < threads; ++thread)
    {
        mThreads.emplace_back(std::make_unique<std::thread>(
            [&ioContext, &thread]()
            {
                getServed(ioContext, thread);
            } ));
    }

    for(auto& thread : mThreads)
    {
        thread->join();
    }

    std::cout << "done" << std::endl;
}
