#include <boost/asio.hpp>

#include <signal.h>
#include <boost/asio/signal_set.hpp>

#include <thread>
#include <iostream>
#include <string>
#include <array>
#include <algorithm>

/// UDP : connectionLESS

/// a server which is :
/// - SYNCHRONOUS
/// - reads the message from the client
/// - sends this message back

/// ===> does not scale


namespace
{
std::atomic_bool stop{false};

class Server
{
public:
    Server(
            boost::asio::io_context& ioContext,
            unsigned short service) :
        mIoContext{ioContext},
        mService{service},
        mStop{false}
    {
    }

    void serve() /// can only stop when a message has arrived and we loop again => drawback of synchronous
    {
        boost::asio::ip::udp::socket sock{mIoContext, {boost::asio::ip::udp::v4(), mService} };
        std::array<char, 70'000> readBuffer;/// Max IP datagram size is 65535 bytes
        while(!mStop)
        {
            boost::asio::ip::udp::endpoint sender;
            const auto bytes = sock.receive_from(
                        boost::asio::buffer(readBuffer),
                        sender);

            std::string message;
            message.resize(bytes);
            std::copy(readBuffer.cbegin(), readBuffer.cbegin() + bytes, message.begin());

            std::cout << "received message of size " << bytes << std::endl;
            std::cout << "from " << sender << std::endl;
            std::cout << message << std::endl;

            std::this_thread::sleep_for(std::chrono::seconds(5)); // processing takes time

            sock.send_to(boost::asio::buffer(message), sender);
        }
    }

    void stop()
    {
        mStop = true;
    }

private:
    boost::asio::io_context& mIoContext;
    unsigned short mService;
    std::atomic_bool mStop;
};

void runServer(Server& server)
{
	server.serve();
}

void handlerSignals(
        const boost::system::error_code& /*erc*/,
        int signal)
{
    std::cout << "signal " << signal << " occurred. Time to stop the application." <<std::endl;
    stop = true;
}

void asyncSignal(boost::asio::io_context& ioContext)
{
    boost::asio::signal_set signals{ioContext, SIGINT, SIGTERM};

    signals.async_wait(handlerSignals);
    ioContext.run();
}

} // namespace

int main()
{
    boost::asio::io_context ioContextSignal;
    boost::asio::io_context ioContextServer;

    std::thread t1{asyncSignal, std::ref(ioContextSignal)};

    Server server{ioContextServer, 9242};
    std::thread t2{runServer, std::ref(server)};

    t1.join(); /// wait on stop signal

    std::cout << "time stop go home, tell the server to stop" << std::endl;
    server.stop();

    t2.join();

    std::cout << "done" << std::endl;
}
