#include <boost/asio.hpp>

#include <thread>
#include <iostream>
#include <string>

/// we will intertact with the synchronous server of Asio11-UdpEchoServer1
/// we will make multiple threads that request and get served

namespace
{

const boost::asio::ip::udp::endpoint ep{boost::asio::ip::make_address("127.0.0.1"), 9242};

void getServed(
        boost::asio::io_context& ioContext,
        int id)
{
    std::this_thread::sleep_for(std::chrono::seconds(1)); /// so all thread are booted up and can start to fight for the cpu

	const std::string message = "request from client " + std::to_string(id) + " .\n";

	boost::asio::ip::udp::socket sock{ioContext, {boost::asio::ip::udp::v4(), 0} };

	sock.send_to(boost::asio::buffer(message), ep);

	std::array<char, 70'000> readBuffer;

    boost::asio::ip::udp::endpoint sender;
    const auto bytes = sock.receive_from(
                boost::asio::buffer(readBuffer),
                sender);

    std::string answer;
    answer.resize(bytes);
    std::copy(readBuffer.cbegin(), readBuffer.cbegin() + bytes, answer.begin());

    std::cout << "server replied : " << answer << std::endl;
    std::cout << "this is indeed echo of our message : " << (message == answer)<< std::endl;

	std::cout << "client " << id << " got served" << std::endl << std::endl;
}

} // namespace

int main()
{
    boost::asio::io_context ioContext;


    std::vector<std::unique_ptr<std::thread>> mThreads;

    const int threads{10};

    for(auto thread = 0; thread < threads; ++thread)
    {
        mThreads.emplace_back(std::make_unique<std::thread>(
            [&ioContext, &thread]()
            {
                getServed(ioContext, thread);
            } ));
    }

    for(auto& thread : mThreads)
    {
        thread->join();
    }

    std::cout << "done" << std::endl;
}
