#include <iostream>
#include <boost/asio/ip/address.hpp>

namespace
{

void demoIpV4Address(const boost::asio::ip::address_v4& address)
{
	std::cout << address << std::endl;

	std::cout << "Class A : " << std::boolalpha << address.is_class_a() << std::endl;
	std::cout << "Class B : " << std::boolalpha << address.is_class_b() << std::endl;
	std::cout << "Class C : " << std::boolalpha << address.is_class_c() << std::endl;

	std::cout << "Multicast : " << std::boolalpha << address.is_multicast() << std::endl;
	std::cout << "Loopback : " << std::boolalpha << address.is_loopback() << std::endl;

	std::cout << "Numeric Value : " << address.to_uint() << std::endl;
	std::cout << "String Value : " << address.to_string() << std::endl;

	const auto bytes = address.to_bytes();  /// boost::asio::ip::address_v4::bytes_type
	for(auto byte : bytes)
	{
		std::cout << static_cast<int>(byte) << "/";
	}
	std::cout << std::endl;

	std::cout << std::endl << std::endl;
}

void demoIpAddress(const boost::asio::ip::address& address)
{
	std::cout << address << std::endl;

	std::cout << "Multicast : " << std::boolalpha << address.is_multicast() << std::endl;
	std::cout << "Loopback : " << std::boolalpha << address.is_loopback() << std::endl;
	std::cout << "Is IPv4 : " << std::boolalpha << address.is_v4() << std::endl;
	std::cout << "Is IPv6 : " << std::boolalpha << address.is_v6() << std::endl;
	if(address.is_v4())
	{
		std::cout << "Ipv4 format : " << address.to_v4() << std::endl;
	}
	else if(address.is_v6())
	{
		std::cout << "Ipv6 format : " << address.to_v6() << std::endl;
	}
	std::cout << std::endl << std::endl;
}

} // namespace

int main()
{
	{
		boost::asio::ip::address_v4::bytes_type address;
		address[0] = 127;
		address[1] = 0;
		address[2] = 0;
		address[3] = 1;
		demoIpV4Address(boost::asio::ip::address_v4(address));
	}

	{
		boost::asio::ip::address_v4::bytes_type address{{192, 168, 0, 1}};
		demoIpV4Address(boost::asio::ip::address_v4(address));
	}

	{
		boost::asio::ip::address_v4::bytes_type address{{192, 168, 0, 10}};
		demoIpAddress(boost::asio::ip::address_v4(address));
	}

	{
		boost::asio::ip::address_v4::bytes_type address{{239, 1, 0, 10}};
		demoIpAddress(boost::asio::ip::address_v4(address));
	}

	{
		boost::asio::ip::address_v4 address = boost::asio::ip::address_v4::from_string("10.0.0.1");
		demoIpV4Address(address);
	}

	/// make address is the new way !!! from string is deprecated (on boost::asio::ip::addressaddress !!)
	{
		boost::asio::ip::address address = boost::asio::ip::address::from_string("10.0.0.10");
		demoIpAddress(address);
	}

	{
		boost::asio::ip::address address = boost::asio::ip::make_address("242.0.0.242");
		demoIpAddress(address);
	}
}
