#include <iostream>
#include <boost/asio.hpp>

/// resolvers for : tcp / udp / icmp

/// new style : no more queries

/// the resolvers resolve()s

namespace
{

void ResolveTcp(const std::string& host, const std::string& service)
{
	boost::asio::io_context ioService;
	boost::asio::ip::tcp::resolver res(ioService);

	boost::system::error_code ec;
	boost::asio::ip::tcp::resolver::results_type results = res.resolve(host, service, ec);
	if(ec)
	{
		std::cout << "Oops : " << ec.message() << std::endl;
		return;
	}

///	for(auto it = results.cbegin(); it != results.cend(); ++it) or we can do better : range based for

	for(const auto& result : results)
	{
	    const auto& endPoint{result.endpoint()};
		std::cout << "End point : " << endPoint <<  "      " <<
                endPoint.address().to_string() <<  " " <<
                endPoint.port() << "  " <<
                ((endPoint.protocol() == boost::asio::ip::tcp::v4()) ? "v4" : "v6") << std::endl;
	}
}

void ResolveTcpThrow(const std::string& host, const std::string& service)
{
	boost::asio::io_context ioService;
	boost::asio::ip::tcp::resolver res{ioService};

	try
	{
        const auto& results = res.resolve(host, service);
	}
	catch(const std::exception& exc)
	{
	    std::cout  << "Exception : " << exc.what() << std::endl;
	}
}

} // namespace

int main()
{
	ResolveTcp("localhost", "");
	std::cout << std::endl;

	ResolveTcp("localhost", "80");
	std::cout << std::endl;

	ResolveTcp("localhost", "http");
	std::cout << std::endl;

	ResolveTcp("localhost", "ftp");
	std::cout << std::endl;

	ResolveTcp("www.google.com", "http");
	std::cout << std::endl;

	ResolveTcp("www.nope.nope", "");
	std::cout << std::endl;


	ResolveTcpThrow("www.nope.nope", "");
	std::cout << std::endl;
}
