#include <iostream>
#include <boost/asio.hpp>

/// resolvers for : tcp / udp / icmp

/// the resolvers resolves queries
/// and the query is now the task to carry out and to call our handler at completion

namespace
{

void handler(
        const boost::system::error_code& ec,
        boost::asio::ip::tcp::resolver::iterator it)   /// boost::asio::ip::tcp::resolver::results_type results
{
	if(ec)
	{
		std::cout << "Oops : " << ec.message() << std::endl;
		return;
	}
	boost::asio::ip::tcp::resolver::iterator end;
	while(it != end)
	{
	    const auto& endPoint{it->endpoint()};
		std::cout << "End point : " << endPoint <<  "      " <<
                endPoint.address().to_string() <<  " " <<
                endPoint.port() << "  " <<
                ((endPoint.protocol() == boost::asio::ip::tcp::v4()) ? "v4" : "v6") << std::endl;
		++it;
	}
}

void ResolveTcp(const std::string& host, const std::string& service)
{
	boost::asio::io_service ioService;
	boost::asio::ip::tcp::resolver res{ioService};
	boost::asio::ip::tcp::resolver::query query{host, service};

	res.async_resolve(query, handler);

	ioService.run();
}

} // namespace

int main()
{
	ResolveTcp("localhost", "");
	std::cout << std::endl;

	ResolveTcp("localhost", "80");
	std::cout << std::endl;

	ResolveTcp("localhost", "http");
	std::cout << std::endl;

	ResolveTcp("localhost", "ftp");
	std::cout << std::endl;

	ResolveTcp("www.google.com", "http");
	std::cout << std::endl;

	ResolveTcp("www.nope.nope", "");
	std::cout << std::endl;

    /// no more throw variant
}
