#include <iostream>
#include <boost/asio.hpp>

/// resolvers for : tcp / udp / icmp

/// the resolvers resolves queries

/// we do it in a synchronous way in these examples

namespace
{

void ResolveTcp(const std::string& host, const std::string& service)
{
	boost::asio::io_service ioService;

	boost::asio::ip::tcp::resolver res{ioService};

	/// host name and service name (the latter can also be string representation of a port number)
	boost::asio::ip::tcp::resolver::query query{host, service};

	/// resolve the query ==> result goes into an iterator (since can be more then 1 resolution => multiple ip addresses)
	boost::system::error_code ec;
	boost::asio::ip::tcp::resolver::iterator it = res.resolve(query, ec);
	if(ec)
	{
		std::cout << "Oops : " << ec.message() << std::endl;
		return;
	}

	boost::asio::ip::tcp::resolver::iterator end;
	while(it != end)
	{
	    const auto& endPoint{it->endpoint()};
		std::cout << "End point : " << endPoint <<  "      " <<
                endPoint.address().to_string() <<  " " <<
                endPoint.port() << "  " <<
                ((endPoint.protocol() == boost::asio::ip::tcp::v4()) ? "v4" : "v6") << std::endl;
		++it;
	}

}

void ResolveTcpThrow(const std::string& host, const std::string& service)
{
	boost::asio::io_service ioService;
	boost::asio::ip::tcp::resolver res(ioService);
	boost::asio::ip::tcp::resolver::query query{host, service};

	try
	{
        boost::asio::ip::tcp::resolver::iterator it = res.resolve(query);
	}
	catch(const std::exception& exc)
	{
	    std::cout  << "Exception : " << exc.what() << std::endl;
	}
}

} // namespace

int main()
{
	ResolveTcp("localhost", "");
	std::cout << std::endl;

	ResolveTcp("localhost", "80");
	std::cout << std::endl;

	ResolveTcp("localhost", "http");
	std::cout << std::endl;

	ResolveTcp("localhost", "ftp");
	std::cout << std::endl;

	ResolveTcp("www.google.com", "http");
	std::cout << std::endl;

	ResolveTcp("www.nope.nope", "");
	std::cout << std::endl;


	ResolveTcpThrow("www.nope.nope", "");
	std::cout << std::endl;
}
