#include <iostream>
#include <boost/asio.hpp>

/// we want to connect to the http of google

/// we need a SOCKET for that purpose

/// sockets of type : tcp / udp / icmp


namespace
{

auto resolveTcp(const std::string& host, const std::string& service)
{
	boost::asio::io_context ioService;
	boost::asio::ip::tcp::resolver res{ioService};

    return res.resolve(host, service);
}

} // namespace

int main()
{
	const auto& results =  resolveTcp("www.google.com", "http");
	if(results.empty())
    {
        std::cout << " could not resolve, it stops here";
        return 0;
    }


    /// let's just use the first result and use that to connect
    {
        boost::asio::io_context ioService;
        boost::asio::ip::tcp::socket socket{ioService};

        boost::system::error_code ec;
        socket.connect(*results.cbegin(), ec);
        if(ec)
        {
            std::cout << "Connect failed : " << ec.message() << std::endl;
            return 0;
        }
        std::cout << "we are connected : " << std::endl;
        std::cout << "remote : " << socket.remote_endpoint() << std::endl;
        std::cout << "local : " << socket.local_endpoint() << std::endl;
    }

    std::cout << std::endl;

    /// let's not deal with the amount of results, and pass them all to connect which will use them one by one till one works
    {
        boost::asio::io_context ioService;
        boost::asio::ip::tcp::socket socket{ioService};

        boost::system::error_code ec;
        boost::asio::connect(socket, results, ec);
        if(ec)
        {
            std::cout << "Connect failed : " << ec.message() << std::endl;
            return 0;
        }
        std::cout << "we are connected" << std::endl;
        std::cout << "remote : " << socket.remote_endpoint() << std::endl;
        std::cout << "local : " << socket.local_endpoint() << std::endl;
    }
}
