#include <iostream>
#include <thread>
#include <functional>
#include <memory>
#include <boost/asio.hpp>

/// we want to connect to the http of google

/// we need a SOCKET for that purpose

/// sockets of type : tcp / udp / icmp


namespace
{

void connectHandler(
        std::shared_ptr<boost::asio::ip::tcp::socket> socket,
        const boost::system::error_code& ec,
        boost::asio::ip::tcp::endpoint endPoint)
{
    if(ec)
    {
        std::cout << "Connect failed : " << ec.message() << std::endl;
        return;
    }
    std::cout << "we are connected" << std::endl;
    std::cout << "with : " << endPoint << std::endl;
    std::cout << "remote : " << socket->remote_endpoint() << std::endl;
    std::cout << "local : " << socket->local_endpoint() << std::endl;
}

void resolveHandler(
        boost::asio::io_context& ioService,
        const boost::system::error_code& /*ec*/,
        boost::asio::ip::tcp::resolver::results_type results)
{
        auto socket = std::make_shared<boost::asio::ip::tcp::socket>(ioService);
        boost::asio::async_connect(*socket, results, std::bind(connectHandler, socket, std::placeholders::_1, std::placeholders::_2));
}


void connectToGoogle()
{
	boost::asio::io_context ioService;

	boost::asio::ip::tcp::resolver res{ioService};
	res.async_resolve("www.google.com", "http", std::bind(resolveHandler, std::ref(ioService), std::placeholders::_1, std::placeholders::_2));

	ioService.run();
}

} // namespace

int main()
{
    std::thread t{connectToGoogle};

    for(auto idx = 0; idx < 2; ++idx)
    {
        std::cout << "heavy work being done in main thread."  << std::endl;
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }

    t.join();
}
