#include <boost/asio.hpp>

#include <signal.h>
#include <boost/asio/signal_set.hpp>

#include <thread>
#include <iostream>

/// when we play the role of a server,
/// we are serving
/// we are providing a service
///
///     - on some interfaces (multi homed aspect)
///     - on a specific port (service)
///
/// in case we offer the service on all our interfaces,
/// then the ip address of the local endpoint does not matter ==> any, or the default ip address that is


/// here we will Asynchronously accept connections (and not do anything useful with those connections)

/// connections are accepted by a ACCEPTOR

/// use Asio7-ConnectorTest to challenge this code

namespace
{
std::atomic_bool stop{false};


class Acceptor
{
public:
    Acceptor(
            boost::asio::io_context& ioContext,
            unsigned short service) :
        mSocket{ioContext},
        mAcceptor{ioContext, {boost::asio::ip::tcp::v4(), service}},
        mRemote{},
        mStop{false}
    {
    }

    void start()
    {           /// needs to be a mSocket per client that lives long enough ==> shared_ptr
        mAcceptor.async_accept(mSocket, mRemote, std::bind(&Acceptor::acceptHandler, this, std::placeholders::_1));
    }

    void stop()
    {
        mStop = true;
        mAcceptor.cancel();
    }
private:
    void acceptHandler(const boost::system::error_code& ec)
    {
        if(ec)
        {
            std::cout << "Accept failed : " << ec.message() << std::endl;
            std::cout << std::endl;
        }
        else
        {
            std::cout << "we accept connection" << std::endl;
            std::cout << "from : " << mRemote << std::endl;
            std::cout << "remote : " << mSocket.remote_endpoint() << std::endl;
            std::cout << "local : " << mSocket.local_endpoint() << std::endl;
            std::cout << std::endl;
        }
        if(mStop)
        {
            return;
        }

        mSocket.close(); /// let's close it so we can reuse it for a new asycn accept (otherwise error because it is already open)

        mAcceptor.async_accept(mSocket, mRemote, std::bind(&Acceptor::acceptHandler, this, std::placeholders::_1));
    }

    boost::asio::ip::tcp::socket mSocket;
    boost::asio::ip::tcp::acceptor mAcceptor;
    boost::asio::ip::tcp::endpoint mRemote;
    std::atomic_bool mStop;
};


void handleConnections(
        Acceptor& acceptor,
        boost::asio::io_context& ioContext)
{
	acceptor.start();

	ioContext.run();
}

void handlerSignals(
        const boost::system::error_code& /*erc*/,
        int signal)
{
    std::cout << "signal " << signal << " occurred. Time to stop the application." <<std::endl;
    stop = true;
}

void asyncSignal(boost::asio::io_context& ioContext)
{
    boost::asio::signal_set signals{ioContext, SIGINT, SIGTERM};

    signals.async_wait(handlerSignals);
    ioContext.run();
}

} // namespace

int main()
{
    boost::asio::io_context ioContextSignal;
    boost::asio::io_context ioContextAcceptor;

    std::thread t1{asyncSignal, std::ref(ioContextSignal)};

    Acceptor acceptor{ioContextAcceptor, 8242};
    std::thread t2{handleConnections, std::ref(acceptor), std::ref(ioContextAcceptor)};

    t1.join(); /// wait on stop signal

    std::cout << "time stop go home, tell the acceptor to stop" << std::endl;
    acceptor.stop();

    t2.join();

    std::cout << "done" << std::endl;
}
