#include <boost/asio.hpp>

#include <signal.h>
#include <boost/asio/signal_set.hpp>

#include <thread>
#include <iostream>

/// use Asio7-ConnectorTest to challenge this code

namespace
{
std::atomic_bool stop{false};


class Acceptor
{
public:
    Acceptor(
            boost::asio::io_context& ioContext,
            unsigned short service) :
        mAcceptor{ioContext, {boost::asio::ip::tcp::v4(), service}},
        mStop{false}
    {
    }

    void start()
    {
        mAcceptor.async_accept(std::bind(&Acceptor::acceptHandler, this, std::placeholders::_1, std::placeholders::_2)); // the overload to pass in the endpoint is also possible
    }

    void stop()
    {
        mStop = true;
        mAcceptor.cancel();
    }
private:
    void acceptHandler(
            const boost::system::error_code& ec,
            boost::asio::ip::tcp::socket socket)
    {
        if(ec)
        {
            std::cout << "Accept failed : " << ec.message() << std::endl;
            std::cout << std::endl;
        }
        else
        {
            std::cout << "we accept connection" << std::endl;
            std::cout << "remote : " << socket.remote_endpoint() << std::endl;
            std::cout << "local : " << socket.local_endpoint() << std::endl;
            std::cout << std::endl;
        }
        if(mStop)
        {
            return;
        }

        mAcceptor.async_accept(std::bind(&Acceptor::acceptHandler, this, std::placeholders::_1, std::placeholders::_2));
    }

    boost::asio::ip::tcp::acceptor mAcceptor;
    std::atomic_bool mStop;
};


void handleConnections(
        Acceptor& acceptor,
        boost::asio::io_context& ioContext)
{
	acceptor.start();

	ioContext.run();
}

void handlerSignals(
        const boost::system::error_code& /*erc*/,
        int signal)
{
    std::cout << "signal " << signal << " occurred. Time to stop the application." <<std::endl;
    stop = true;
}

void asyncSignal(boost::asio::io_context& ioContext)
{
    boost::asio::signal_set signals{ioContext, SIGINT, SIGTERM};

    signals.async_wait(handlerSignals);
    ioContext.run();
}

} // namespace

int main()
{
    boost::asio::io_context ioContextSignal;
    boost::asio::io_context ioContextAcceptor;

    std::thread t1{asyncSignal, std::ref(ioContextSignal)};

    Acceptor acceptor{ioContextAcceptor, 8242};
    std::thread t2{handleConnections, std::ref(acceptor), std::ref(ioContextAcceptor)};

    t1.join(); /// wait on stop signal

    std::cout << "time stop go home, tell the acceptor to stop" << std::endl;
    acceptor.stop();

    t2.join();

    std::cout << "done" << std::endl;
}
