#include <boost/asio.hpp>

#include <signal.h>
#include <boost/asio/signal_set.hpp>

#include <thread>
#include <iostream>

/// when we play the role of a server,
/// we are serving
/// we are providing a service
///
///     - on some interfaces (multi homed aspect)
///     - on a specific port (service)
///
/// in case we offer the service on all our interfaces,
/// then the ip address of the local endpoint does not matter ==> any, or the default ip address that is


/// here we will synchronously accept connections (and not do anything useful with those connections)

/// connections are accepted by a ACCEPTOR

/// use Asio7-ConnectorTest to challenge this code

/// =============> "nice" shutdown problem

namespace
{
std::atomic_bool stop{false};

void handleConnections(boost::asio::io_context& ioContext)
{
    boost::asio::ip::tcp::endpoint localEndPoint{boost::asio::ip::tcp::v4(), 8242};

    /// INSTANTIATE and OPEN acceptor socket
    boost::asio::ip::tcp::acceptor acceptor{ioContext, localEndPoint.protocol()};

    /// BIND it to the endpoint
    acceptor.bind(localEndPoint);

    /// START listening
    acceptor.listen(30); /// queue size of 30

    while(!stop)
    {
        boost::asio::ip::tcp::socket socket{ioContext};
        boost::asio::ip::tcp::endpoint endpointRemote{};
        acceptor.accept(socket, endpointRemote);
        std::cout << "something got accepted from " << endpointRemote << std::endl;
    }
    std::cout << "we are out" << std::endl;
}

void handlerSignals(
        const boost::system::error_code& /*erc*/,
        int signal)
{
    std::cout << "signal " << signal << " occurred. Time to stop the application." <<std::endl;
    stop = true;
}

void asyncSignal(boost::asio::io_context& ioContext)
{
    boost::asio::signal_set signals{ioContext, SIGINT, SIGTERM};

    signals.async_wait(handlerSignals);
    ioContext.run();
}

} // namespace

int main()
{
    boost::asio::io_context ioContextSignal;
    boost::asio::io_context ioContextAcceptor;

    std::thread t1{asyncSignal, std::ref(ioContextSignal)};
    std::thread t2{handleConnections, std::ref(ioContextAcceptor)};

    t1.join(); /// wait on stop signal

    std::cout << "time stop go home, tell the acceptor context to stop" << std::endl;

    ioContextAcceptor.stop(); /// to stop block/wait on a new connection : WE WISH
    t2.join();

    std::cout << "done" << std::endl;
}
