#include <iostream>
#include <thread>
#include <functional>
#include <memory>
#include <boost/asio.hpp>

namespace
{


class ResolveAndConnect
{
public:
    ResolveAndConnect(
            boost::asio::io_context& ioContext,
            const std::string& host,
            const std::string& service) :
        mIoContext{ioContext},
        mSocket{mIoContext},
        mResolver{mIoContext},
        mHost{host},
        mService{service}
    {
    }

    void doIt()
    {
        mResolver.async_resolve(mHost, mService, std::bind(&ResolveAndConnect::resolveHandler, this, std::placeholders::_1, std::placeholders::_2));
    }

private:
    void resolveHandler(
            const boost::system::error_code& ec,
            boost::asio::ip::tcp::resolver::results_type results)
    {
        if(ec)
        {
            std::cout << "resolve failed : " << ec.message() << std::endl;
            return;
        }
        boost::asio::async_connect(mSocket, results, std::bind(&ResolveAndConnect::connectHandler, this, std::placeholders::_1, std::placeholders::_2));
    }

    void connectHandler(
            const boost::system::error_code& ec,
            boost::asio::ip::tcp::endpoint endPoint)
    {
        if(ec)
        {
            std::cout << "Connect failed : " << ec.message() << std::endl;
            return;
        }
        std::cout << "we are connected" << std::endl;
        std::cout << "with : " << endPoint << std::endl;
        std::cout << "remote : " << mSocket.remote_endpoint() << std::endl;
        std::cout << "local : " << mSocket.local_endpoint() << std::endl;
        std::cout << std::endl;
    }

    boost::asio::io_context& mIoContext;
    boost::asio::ip::tcp::socket mSocket;
    boost::asio::ip::tcp::resolver mResolver;
    std::string mHost;
    std::string mService;
};


void connectToserver()
{
	boost::asio::io_context ioContext;

	ResolveAndConnect rac1{ioContext, "localhost", "8242"};
	rac1.doIt();

	ResolveAndConnect rac2{ioContext, "localhost", "8242"};
	rac2.doIt();

	ioContext.run();
}

} // namespace

int main()
{
    std::thread t{connectToserver};

    for(auto idx = 0; idx < 2; ++idx)
    {
        std::cout << "heavy work being done in main thread."  << std::endl;
        std::this_thread::sleep_for(std::chrono::seconds(1));
    }

    t.join();
}
