#include <iostream>
#include <thread>
#include <functional>
#include <boost/asio.hpp>

/// sending synchronously
/// use Asio8-ReadAndIgnoreTester for example as the party to connect to

namespace
{

void sending(boost::asio::ip::tcp::socket& socket, int payload);

class ResolveAndConnect
{
public:
    ResolveAndConnect(
            boost::asio::io_context& ioContext,
            const std::string& host,
            const std::string& service) :
        mIoContext{ioContext},
        mSocket{mIoContext},
        mResolver{mIoContext},
        mHost{host},
        mService{service}
    {
    }

    void doIt()
    {
        mResolver.async_resolve(mHost, mService, std::bind(&ResolveAndConnect::resolveHandler, this, std::placeholders::_1, std::placeholders::_2));
    }

private:
    void resolveHandler(
            const boost::system::error_code& ec,
            boost::asio::ip::tcp::resolver::results_type results)
    {
        if(ec)
        {
            std::cout << "resolve failed : " << ec.message() << std::endl;
            return;
        }
        boost::asio::async_connect(mSocket, results, std::bind(&ResolveAndConnect::connectHandler, this, std::placeholders::_1, std::placeholders::_2));
    }

    void connectHandler(
            const boost::system::error_code& ec,
            boost::asio::ip::tcp::endpoint /*endPoint*/)
    {
        if(ec)
        {
            std::cout << "Connect failed : " << ec.message() << std::endl;
            return;
        }
//        sending(mSocket, 1);
//        sending(mSocket, 10);
        sending(mSocket, 4000);
//        sending(mSocket, 50000);
    }

    boost::asio::io_context& mIoContext;
    boost::asio::ip::tcp::socket mSocket;
    boost::asio::ip::tcp::resolver mResolver;
    std::string mHost;
    std::string mService;
};


void connectToserver()
{
	boost::asio::io_context ioContext;

	ResolveAndConnect rac1{ioContext, "localhost", "8242"};
	rac1.doIt();
	ioContext.run();
}

std::string constructPayload(int times)
{
    std::string payload;
    for(auto i = 1; i <= times; ++i)
    {
        for(char c = 'a'; c <= 'z'; ++c)
        {
            payload += std::string(1, c) + std::to_string(i);
        }
        payload += ";";
    }
    return payload + "\n";
}

/// we will send synchronously
void sending(boost::asio::ip::tcp::socket& socket, int payload)
{
    const auto message{constructPayload(payload)};
    const auto written = socket.write_some(boost::asio::buffer(message));  /// or socket.send()
    std::cout << "message size is : " << message.size() << std::endl;
    std::cout << "we have sent : " << written << std::endl;
    if(message.size() != written)
    {
        std::cout << "not all got transferred at once, so we need to continue calling until all is processed" << std::endl;

        /// we are not dealing with the rest in this example
    }
}

} // namespace

int main()
{
    std::thread t{connectToserver};
    t.join();
}
