#include <iostream>
#include <thread>
#include <functional>
#include <boost/asio.hpp>

/// sending asynchronously
/// use Asio8-ReadAndIgnoreTester for example as the party to connect to

namespace
{

std::string constructPayload(int times)
{
    std::string payload;
    for(auto i = 1; i <= times; ++i)
    {
        for(char c = 'a'; c <= 'z'; ++c)
        {
            payload += std::string(1, c) + std::to_string(i);
        }
        payload += ";";
    }
    return payload + "\n";
}

class ResolveAndConnectAndWrite
{
public:
    ResolveAndConnectAndWrite(
            boost::asio::io_context& ioContext,
            const std::string& host,
            const std::string& service) :
        mIoContext{ioContext},
        mSocket{mIoContext},
        mResolver{mIoContext},
        mHost{host},
        mService{service},
        mMessage{}
    {
    }

    void doIt()
    {
        mResolver.async_resolve(mHost, mService, std::bind(&ResolveAndConnectAndWrite::resolveHandler, this, std::placeholders::_1, std::placeholders::_2));
    }

private:
    void resolveHandler(
            const boost::system::error_code& ec,
            boost::asio::ip::tcp::resolver::results_type results)
    {
        if(ec)
        {
            std::cout << "resolve failed : " << ec.message() << std::endl;
            return;
        }
        boost::asio::async_connect(mSocket, results, std::bind(&ResolveAndConnectAndWrite::connectHandler, this, std::placeholders::_1, std::placeholders::_2));
    }

    void connectHandler(
            const boost::system::error_code& ec,
            boost::asio::ip::tcp::endpoint /*endPoint*/)
    {
        if(ec)
        {
            std::cout << "Connect failed : " << ec.message() << std::endl;
            return;
        }

        /// we will send asynchronously
        mMessage = constructPayload(4000);
        boost::asio::async_write(mSocket, boost::asio::buffer(mMessage),
                    std::bind(&ResolveAndConnectAndWrite::writeCompleted, this, std::placeholders::_1, std::placeholders::_2));
    }

    void writeCompleted(
            const boost::system::error_code& ec,
            size_t bytes)
    {
        if(ec)
        {
            std::cout << "Write failed : " << ec.message() << std::endl;
        }
        else
        {
            std::cout << "wrote successfully " << bytes << " bytes" << std::endl;
        }
    }

    boost::asio::io_context& mIoContext;
    boost::asio::ip::tcp::socket mSocket;
    boost::asio::ip::tcp::resolver mResolver;
    std::string mHost;
    std::string mService;
    std::string mMessage;
};


void connectToserver()
{
	boost::asio::io_context ioContext;

	ResolveAndConnectAndWrite rac1{ioContext, "localhost", "8242"};
	rac1.doIt();
	ioContext.run();
}

} // namespace

int main()
{
    std::thread t{connectToserver};
    t.join();
}
